@@ -1,8 +1,8 @@
 #/*##########################################################################
-# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2024 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
-# the ESRF by the Software group.
+# the ESRF.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 # THE SOFTWARE.
 #
 #############################################################################*/
-__author__ = "V.A. Sole - ESRF Data Analysis"
+__author__ = "V.A. Sole"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
@@ -34,8 +34,10 @@ import weakref
 import gc
 import h5py
 import logging
+_logger = logging.getLogger(__name__)
 
 from PyMca5.PyMcaGui import PyMcaQt as qt
+from PyMca5.PyMcaGui.io import PyMcaFileDialogs
 from PyMca5.PyMcaCore import NexusTools
 safe_str = qt.safe_str
 
@@ -51,15 +53,19 @@ from . import HDF5McaTable
 from . import QNexusWidgetActions
 try:
     from . import Hdf5NodeView
-except ImportError:
+except Exception:
+    # prevent crushing due to other silx-related errors
+    _logger.info("Cannot use silx Hdf5NodeView")
     from . import HDF5DatasetTable
     Hdf5NodeView = None
+try:
+    from . import DataViewerSelector
+    SINGLE_ENABLED = True
+except Exception:
+    _logger.debug("Cannot import DataViewerSelector")
+    SINGLE_ENABLED = False
 from PyMca5.PyMcaIO import ConfigDict
-if "PyMcaDirs" in sys.modules:
-    from PyMca5 import PyMcaDirs
-
-_logger = logging.getLogger(__name__)
-
+from PyMca5 import PyMcaDirs
 
 class Buttons(qt.QWidget):
 
@@ -87,7 +93,12 @@ class Buttons(qt.QWidget):
                 self.mainLayout.addWidget(button, row, col)
                 self.buttonGroup.addButton(button)
                 self.buttonList.append(button)
-        self.buttonGroup.buttonClicked[int].connect(self.emitSignal)
+        if hasattr(self.buttonGroup, "idClicked"):
+            self.buttonGroup.idClicked[int].connect(self.emitSignal)
+        else:
+            # deprecated
+            _logger.debug("Using deprecated signal")
+            self.buttonGroup.buttonClicked[int].connect(self.emitSignal)
 
     def emitSignal(self, idx):
         button = self.buttonGroup.button(idx)
@@ -108,6 +119,7 @@ class QNexusWidget(qt.QWidget):
         self._oldCntSelection = None
         self._cntList = []
         self._aliasList = []
+        self._shapeList = []
         self._autoCntList = []
         self._autoAliasList = []
         self._defaultModel = HDF5Widget.FileModel()
@@ -118,10 +130,20 @@ class QNexusWidget(qt.QWidget):
         self._dir = None
         self._lastAction = None
         self._lastEntry = None
+        self._lastMcaSelection = None
+        self._lastCntSelection = None
         self._mca = mca
         self._BUTTONS = buttons
         self.build()
 
+    def sizeHint(self):
+        originalHint = qt.QWidget.sizeHint(self)
+        if isinstance(self.parent(), qt.QDialog):
+            return qt.QSize(2 * originalHint.width(),
+                            originalHint.height())
+        else:
+            return qt.QSize(originalHint.width(),
+                            originalHint.height())
     def build(self):
         self.mainLayout = qt.QVBoxLayout(self)
         self.mainLayout.setContentsMargins(5, 5, 5, 0)
@@ -145,9 +167,8 @@ class QNexusWidget(qt.QWidget):
         #Enable 3D
         BUTTONS = self._BUTTONS
         if BUTTONS:
-            if ('PyMca.Object3D' in sys.modules) or \
-               ('Object3D' in sys.modules) or \
-               ('PyMca5.Object3D' in sys.modules):
+            if ('PyMca5.PyMcaGui.pymca.SilxGLWindow' in sys.modules) or \
+               ('PyMca5.PyMca.SilxGLWindow' in sys.modules):
                 self.buttons = Buttons(self, options=['SCAN', 'MCA', '2D', '3D'])
                 self.cntTable.set3DEnabled(True)
                 self.autoTable.set3DEnabled(True)
@@ -161,9 +182,8 @@ class QNexusWidget(qt.QWidget):
             self.mainLayout.addWidget(self.buttons)
         else:
             self.actions = QNexusWidgetActions.QNexusWidgetActions(self)
-            if ('PyMca.Object3D' in sys.modules) or \
-               ('Object3D' in sys.modules) or \
-               ('PyMca5.Object3D' in sys.modules):
+            if ('PyMca5.PyMcaGui.pymca.SilxGLWindow' in sys.modules) or \
+               ('PyMca5.PyMca.SilxGLWindow' in sys.modules):
                 self.actions.set3DEnabled(True)
             else:
                 self.actions.set3DEnabled(False)
@@ -210,7 +230,7 @@ class QNexusWidget(qt.QWidget):
 
     def _counterTableCustomMenuSlot(self, qpoint):
         self.getWidgetConfiguration()
-        self._cntTableMenu.exec_(qt.QCursor.pos())
+        self._cntTableMenu.exec(qt.QCursor.pos())
 
     def _getConfigurationFromFile(self, fname):
         ddict = ConfigDict.ConfigDict()
@@ -224,14 +244,14 @@ class QNexusWidget(qt.QWidget):
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Information)
             msg.setText("File does not contain HDF5 configuration")
-            msg.exec_()
+            msg.exec()
             return None
 
         if 'WidgetConfiguration' not in ddict['HDF5'].keys():
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Information)
             msg.setText("File does not contain HDF5 WidgetConfiguration")
-            msg.exec_()
+            msg.exec()
             return None
 
         ddict =ddict['HDF5']['WidgetConfiguration']
@@ -242,14 +262,14 @@ class QNexusWidget(qt.QWidget):
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Information)
             msg.setText("File does not contain HDF5 counters information")
-            msg.exec_()
+            msg.exec()
             return None
 
         if len(ddict['counters']) != len(ddict['aliases']):
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Critical)
             msg.setText("Number of counters does not match number of aliases")
-            msg.exec_()
+            msg.exec()
             return None
 
         return ddict
@@ -262,6 +282,13 @@ class QNexusWidget(qt.QWidget):
 
         ddict = self._getConfigurationFromFile(fname)
         if ddict is not None:
+            if "shapes" not in ddict:
+                ddict["shapes"] = [None,] * len(ddict["counters"])
+            elif not hasattr(ddict["shapes"], "__len__"):
+                ddict["shapes"] = [ddict["shapes"]]
+            elif len(ddict["shapes"]) != len(ddict["counters"]):
+                ddict["shapes"] = [ddict["shapes"]]
+            assert len(ddict["shapes"]) == len(ddict["counters"])
             self.setWidgetConfiguration(ddict)
 
     def _mergeCounterTableConfiguration(self):
@@ -276,11 +303,13 @@ class QNexusWidget(qt.QWidget):
         current = self.getWidgetConfiguration()
         cntList = ddict['counters']
         aliasList = ddict['aliases']
+        shapeList = ddict['shapes']
         for i in range(len(cntList)):
             cnt = cntList[i]
             if cnt not in current['counters']:
                 current['counters'].append(cnt)
                 current['aliases'].append(aliasList[i])
+                current['shapes'].append(shapeList[i])
 
         self.setWidgetConfiguration(current)
 
@@ -312,6 +341,7 @@ class QNexusWidget(qt.QWidget):
 
     def _deleteSelectedCountersFromTable(self):
         itemList = self.cntTable.selectedItems()
+        _logger.debug("Selected items = %s" % itemList)
         rowList = []
         for item in itemList:
             row = item.row()
@@ -321,6 +351,7 @@ class QNexusWidget(qt.QWidget):
         rowList.sort()
         rowList.reverse()
         current = self.cntTable.getCounterSelection()
+        _logger.debug("current = %s" % current)
         for row in rowList:
             for key in ['x', 'y', 'm']:
                 if row in current[key]:
@@ -330,80 +361,106 @@ class QNexusWidget(qt.QWidget):
         ddict = {}
         ddict['counters'] = []
         ddict['aliases'] = []
+        ddict['shapes'] = []
         for i in range(self.cntTable.rowCount()):
             if i not in rowList:
-                name = safe_str(self.cntTable.item(i, 0).text())
-                alias = safe_str(self.cntTable.item(i, 4).text())
-                ddict['counters'].append(name)
-                ddict['aliases'].append(alias)
+                #name = safe_str(self.cntTable.item(i, 0).text())
+                #alias = safe_str(self.cntTable.item(i, 4).text())
+                ddict['counters'].append(current["cntlist"][i])
+                ddict['aliases'].append(current["aliaslist"][i])
+                ddict['shapes'].append(current["shapelist"][i])
+
+        for i in rowList:
+            del current['cntlist'][i]
+            del current['aliaslist'][i]
+            del current['shapelist'][i]
 
         self.setWidgetConfiguration(ddict)
         self.cntTable.setCounterSelection(current)
 
     def getInputFilename(self):
         if self._dir is None:
-            if "PyMcaDirs" in sys.modules:
-                inidir = PyMcaDirs.inputDir
-            else:
-                inidir = os.getcwd()
+            inidir = PyMcaDirs.inputDir
         else:
             inidir = self._dir
 
         if not os.path.exists(inidir):
             inidir = os.getcwd()
 
-        ret = safe_str(qt.QFileDialog.getOpenFileName(self,
-                                         "Select a .ini file",
-                                         inidir,
-                                         "*.ini"))
+        fileList = PyMcaFileDialogs.getFileList(parent=self,
+                                                filetypelist=["ini files (*.ini)"],
+                                                message="Select a .ini file",
+                                                currentdir=inidir,
+                                                mode="OPEN",
+                                                getfilter=False)
+
+        if len(fileList):
+            ret = fileList[0]
+        else:
+            ret = ""
+
         if len(ret):
             self._dir = os.path.dirname(ret)
-            if "PyMcaDirs" in sys.modules:
-                PyMcaDirs.inputDir = os.path.dirname(ret)
+            PyMcaDirs.inputDir = os.path.dirname(ret)
         return ret
 
     def getOutputFilename(self):
         if self._dir is None:
-            if "PyMcaDirs" in sys.modules:
-                inidir = PyMcaDirs.outputDir
-            else:
-                inidir = os.getcwd()
+            inidir = PyMcaDirs.outputDir
         else:
             inidir = self._dir
 
         if not os.path.exists(inidir):
             inidir = os.getcwd()
 
-        ret = safe_str(qt.QFileDialog.getSaveFileName(self,
-                                         "Select a .ini file",
-                                         inidir,
-                                         "*.ini"))
+        fileList = PyMcaFileDialogs.getFileList(parent=self,
+                                                filetypelist=["ini files (*.ini)"],
+                                                message="Select a .ini file",
+                                                currentdir=inidir,
+                                                mode="SAVE",
+                                                getfilter=False)
+        if len(fileList):
+            ret = fileList[0]
+        else:
+            ret = ""
+
         if len(ret):
             self._dir = os.path.dirname(ret)
-            if "PyMcaDirs" in sys.modules:
-                PyMcaDirs.outputDir = os.path.dirname(ret)
+            PyMcaDirs.outputDir = os.path.dirname(ret)
         return ret
 
     def getWidgetConfiguration(self):
         cntSelection = self.cntTable.getCounterSelection()
-        ddict = {}
+        if hasattr(self, "actions"):
+            ddict =self.actions.getConfiguration()
+        else:
+            ddict = {}
         ddict['counters'] = cntSelection['cntlist']
         ddict['aliases'] = cntSelection['aliaslist']
+        ddict['shapes'] = cntSelection['shapelist']
         return ddict
 
     def setWidgetConfiguration(self, ddict=None):
+        _logger.debug("setWidgetConfiguration %s" % ddict)
         if ddict is None:
             self._cntList = []
             self._aliasList = []
-        else:
+            self._shapeList = []
+        elif "counters" in ddict and len(ddict["counters"]):
             self._cntList = ddict['counters']
             self._aliasList = ddict['aliases']
             if type(self._cntList) == type(""):
                 self._cntList = [ddict['counters']]
             if type(self._aliasList) == type(""):
                 self._aliasList = [ddict['aliases']]
-        self.cntTable.build(self._cntList, self._aliasList)
-        _logger.debug("TODO - Add selection options")
+            if 'shapes' in ddict:
+                self._shapeList = ddict['shapes']
+            else:
+                self._shapeList = [None,] * len(self._cntList)
+        self.cntTable.build(self._cntList, self._aliasList, shapelist=self._shapeList)
+        if hasattr(self, "actions"):
+            if hasattr(self.actions, "setConfiguration"):
+                self.actions.setConfiguration(ddict)
 
     def setDataSource(self, dataSource):
         self.data = dataSource
@@ -462,7 +519,7 @@ class QNexusWidget(qt.QWidget):
                 width = self._widgetDict[self._lastWidgetId].width()
                 height = self._widgetDict[self._lastWidgetId].height()
                 widget.resize(max(150, width), max(150, height))
-            except:
+            except Exception:
                 pass
         self._lastWidgetId = wid
         self._widgetDict[wid] = widget
@@ -484,7 +541,7 @@ class QNexusWidget(qt.QWidget):
                     widget.w = HDF5DatasetTable.HDF5DatasetTable(widget)
                     try:
                         widget.w.setDataset(dataset)
-                    except:
+                    except Exception:
                         _logger.error("Error filling table")
                     widget.addTab(widget.w, 'DataView')
                     widget.setCurrentWidget(widget.w)
@@ -507,7 +564,7 @@ class QNexusWidget(qt.QWidget):
                                              self._showInfoWidgetSlot)
             _hdf5WidgetDatasetMenu.addAction(QString("Copy Path to Clipboard"),
                                              self._copyPathSlot)
-            _hdf5WidgetDatasetMenu.exec_(qt.QCursor.pos())
+            _hdf5WidgetDatasetMenu.exec(qt.QCursor.pos())
         else:
             #handle a right click on a numeric dataset
             _hdf5WidgetDatasetMenu.addAction(QString("Add to selection table"),
@@ -524,7 +581,7 @@ class QNexusWidget(qt.QWidget):
                                                  self._showInfoWidgetSlot)
                 _hdf5WidgetDatasetMenu.addAction(QString("Copy Path to Clipboard"),
                                              self._copyPathSlot)
-                _hdf5WidgetDatasetMenu.exec_(qt.QCursor.pos())
+                _hdf5WidgetDatasetMenu.exec(qt.QCursor.pos())
         self._lastItemDict = None
         return
 
@@ -552,7 +609,7 @@ class QNexusWidget(qt.QWidget):
         try:
             clipboard = qt.QApplication.clipboard()
             clipboard.setText(ddict["name"])
-        except:
+        except Exception:
             _logger.warning("Unsuccessful copy to clipboard")
 
     def _openDataset(self, ddict=None):
@@ -581,14 +638,14 @@ class QNexusWidget(qt.QWidget):
                         width = self._widgetDict[self._lastWidgetId].width()
                         height = self._widgetDict[self._lastWidgetId].height()
                         widget.resize(max(150, width), max(300, height))
-                    except:
+                    except Exception:
                         pass
                 else:
                     try:
                         width = self._widgetDict[ids[-1]].width()
                         height = self._widgetDict[ids[-1]].height()
                         widget.resize(max(150, width), max(300, height))
-                    except:
+                    except Exception:
                         pass
         widget.notifyCloseEventToWidget(self)
         wid = id(widget)
@@ -618,16 +675,19 @@ class QNexusWidget(qt.QWidget):
             return False
 
     def hdf5Slot(self, ddict):
+        _logger.debug("hdf5Slot %s" % ddict)
         entryName = NexusTools.getEntryName(ddict['name'])
         currentEntry = "%s::%s" % (ddict['file'], entryName)
         if (currentEntry != self._lastEntry) and not self._BUTTONS:
             self._lastEntry = None
             cntList = []
-            mcaList = []
+            cntShapeList = []
             aliasList = []
             measurement = None
             scanned = []
+            scannedShapeList = []
             mcaList = []
+            mcaShapeList = []
             if posixpath.dirname(entryName) != entryName:
                 h5file = HDF5Widget.h5open(ddict['file'])
                 try:
@@ -636,35 +696,93 @@ class QNexusWidget(qt.QWidget):
                     scanned = NexusTools.getScannedPositioners(h5file,
                                                                ddict['name'])
                     if measurement is not None:
+                        measurementDatasets = [item for key,item in measurement.items() \
+                                       if self._isNumeric(item)]
                         measurement = [item.name for key,item in measurement.items() \
                                        if self._isNumeric(item)]
+                        try:
+                            # case insensitive sorting of measurement
+                            if sys.version_info > (3, 3):
+                                measurement.sort(key=str.casefold)
+                        except Exception:
+                            _logger.error("Cannot apply sorting %s" % sys.exc_info()[1])
+                        nmeasurement = len(measurement)
+                        cntShapeList = [None] * nmeasurement
+                        for item in measurementDatasets:
+                            if item.name in measurement:
+                                idx = measurement.index(item.name)
+                                if hasattr(item, "shape"):
+                                    shape = item.shape
+                                else:
+                                    shape = (0,)
+                                cntShapeList[idx] = shape
                     if self._mca:
                         mcaList = NexusTools.getMcaList(h5file, entryName)
+                        mcaShapeList = []
+                        for mca in mcaList:
+                            try:
+                                dataset = h5file[mca]
+                            except KeyError:
+                                dataset = h5file[NexusTools.sanitizeFilePath(h5file, 
+                                                                             mca,
+                                                                             entry=entryName)]
+                            if hasattr(dataset, "shape"):
+                                mcaShapeList.append(dataset.shape)
+                            else:
+                                mcaShapeList.append((0,))
+                    if scanned:
+                        scannedShapeList = []
+                        for itemName in scanned:
+                            try:
+                                dataset = h5file[itemName]
+                            except KeyError:
+                                newItemName = NexusTools.sanitizeFilePath(h5file,
+                                                                         itemName,
+                                                                         entry=entryName)
+                                dataset = h5file[newItemName]
+                            if hasattr(dataset, "shape"):
+                                scannedShapeList.append(dataset.shape)
+                            else:
+                                scannedShapeList.append((0,))
                 finally:
                     h5file.close()
                     h5file = None
                     del h5file
+            self._autoCntList = []
+            self._autoCntShapeList = []
             for i in range(len(scanned)):
                 key = scanned[i]
-                cntList.append(key)
+                self._autoCntList.append(key)
+                self._autoCntShapeList.append(scannedShapeList[i])
                 aliasList.append(posixpath.basename(key))
+
             if measurement is not None:
-                for key in measurement:
-                    if key not in cntList:
-                        cntList.append(key)
+                for i in range(len(measurement)):
+                    key = measurement[i]
+                    if key not in self._autoCntList:
+                        self._autoCntList.append(key)
+                        self._autoCntShapeList.append(cntShapeList[i])
                         aliasList.append(posixpath.basename(key))
+
             cleanedCntList = []
-            for key in cntList:
+            for key in self._autoCntList:
                 root = key.split('/')
                 root = "/" + root[1]
                 if len(key) == len(root):
                     cleanedCntList.append(key)
                 else:
                     cleanedCntList.append(key[len(root):])
+
             self._autoAliasList = aliasList
+
             self._autoCntList = cleanedCntList
+
+            _logger.info("building autoTable")
+
             self.autoTable.build(self._autoCntList,
-                                 self._autoAliasList)
+                                 self._autoAliasList,
+                                 selection=self._lastCntSelection,
+                                 shapelist=self._autoCntShapeList)
             currentTab = qt.safe_str(self.tableTab.tabText( \
                                     self.tableTab.currentIndex()))
             if self._mca:
@@ -678,14 +796,14 @@ class QNexusWidget(qt.QWidget):
                     else:
                         cleanedMcaList.append(key[len(root):])
                     mcaAliasList.append(posixpath.basename(key))
-                self.mcaTable.build(cleanedMcaList, mcaAliasList)
+                self.mcaTable.build(cleanedMcaList, mcaAliasList, mcaShapeList)
                 nTabs = self.tableTab.count()
                 if (len(mcaList) > 0) and (nTabs < 3):
                     self.tableTab.insertTab(2, self.mcaTable, "MCA")
                 elif (len(mcaList)==0) and (nTabs > 2):
                     self.tableTab.removeTab(2)
             _logger.debug("currentTab = %s", currentTab)
-            if currentTab != "USER":
+            if currentTab not in ["USER", "AUTO"]:
                 if (len(mcaList) > 0) and (len(cntList) == 0):
                     idx = self.tableTabOrder.index("MCA")
                     self.tableTab.setCurrentIndex(idx)
@@ -708,7 +826,7 @@ class QNexusWidget(qt.QWidget):
                         elif auto == "REPLACE":
                             self._replaceAction()
         if ddict['event'] == "itemDoubleClicked":
-            if ddict['type'] in ['Dataset']:
+            if ddict['type'] in ['Dataset', "TiledDataset"]:
                 currentIndex = self.tableTab.currentIndex()
                 text = safe_str(self.tableTab.tabText(currentIndex))
                 if text.upper() != "USER":
@@ -732,7 +850,12 @@ class QNexusWidget(qt.QWidget):
                             self._aliasList.append(basename)
                         else:
                             self._aliasList.append(cnt)
-                        self.cntTable.build(self._cntList, self._aliasList)
+                        if 'shape' in ddict and 'x' in ddict['shape']:
+                            shape = [int(item) for item in ddict['shape'].split('x')]
+                        else:
+                            shape = None
+                        self._shapeList.append(shape)
+                        self.cntTable.build(self._cntList, self._aliasList, shapelist=self._shapeList)
             elif (ddict['color'] == qt.Qt.blue) and ("silx" in sys.modules):
                 # there is an action to be applied
                 self.showInfoWidget(ddict["file"], ddict["name"], dset=False)
@@ -763,7 +886,11 @@ class QNexusWidget(qt.QWidget):
                                 self._aliasList.append(basename)
                             else:
                                 self._aliasList.append(cnt)
-                            self.cntTable.build(self._cntList, self._aliasList)
+                            try:
+                                self._shapeList.append(dataset.shape)
+                            except Exception:
+                                self._shapeList.append(None)
+                            self.cntTable.build(self._cntList, self._aliasList, shapelist=self._shapeList)
                         return
                 _logger.debug("Unhandled item type: %s", ddict['dtype'])
 
@@ -858,7 +985,7 @@ class QNexusWidget(qt.QWidget):
         text = qt.safe_str(self.tableTab.tabText(self.tableTab.currentIndex()))
         if text.upper() == "USER":
             actions = self.actions.getConfiguration()
-            if len(self.autoTable.getCounterSelection()['y']):
+            if len(self.cntTable.getCounterSelection()['y']):
                 if actions["auto"] == "ADD":
                     self._addAction()
                 elif actions["auto"] == "REPLACE":
@@ -869,7 +996,7 @@ class QNexusWidget(qt.QWidget):
         text = qt.safe_str(self.tableTab.tabText(self.tableTab.currentIndex()))
         if text.upper() == "MCA":
             actions = self.actions.getConfiguration()
-            if len(self.autoTable.getCounterSelection()['y']):
+            if len(self.mcaTable.getMcaSelection()['selectionindex']):
                 if actions["auto"] == "ADD":
                     self._addAction()
                 elif actions["auto"] == "REPLACE":
@@ -904,6 +1031,10 @@ class QNexusWidget(qt.QWidget):
                 #nothing to plot
                 continue
             mcaIdx = 0
+            if filename not in self.data.sourceName:
+                if ("tiled:" + filename) in self.data.sourceName:
+                    filename = "tiled:" + filename 
+
             for yMca in mcaSelection['selectionindex']:
                 sel = {}
                 sel['SourceName'] = self.data.sourceName * 1
@@ -912,7 +1043,7 @@ class QNexusWidget(qt.QWidget):
                 phynxFile  = self.data._sourceObjectList[fileIndex]
                 entryIndex = list(phynxFile["/"].keys()).index(entry[1:])
                 sel['Key']        = "%d.%d" % (fileIndex+1, entryIndex+1)
-                sel['legend']     = os.path.basename(sel['SourceName'][0])+\
+                sel['legend']     = os.path.basename(filename)+\
                                     " " + posixpath.basename(entry) #it was sel['Key']
                 sel['selection'] = {}
                 sel['selection']['sourcename'] = filename
@@ -923,6 +1054,8 @@ class QNexusWidget(qt.QWidget):
                 sel['selection']['key'] = "%d.%d" % (fileIndex+1, entryIndex+1)
                 sel['selection']['mca'] = [yMca]
                 sel['selection']['mcaselectiontype'] = mcaSelection['selectiontype'][mcaIdx]
+                sel['legend'] += " " + mcaSelection['aliaslist'][yMca] + \
+                                 " " + sel['selection']['mcaselectiontype']
                 mcaIdx += 1
                 sel['selection']['mcalist'] = mcaSelection['mcalist']
                 sel['selection']['LabelNames'] = mcaSelection['aliaslist']
@@ -943,18 +1076,26 @@ class QNexusWidget(qt.QWidget):
                 else:
                     entryIndex = list(phynxFile["/"].keys()).index(entry[1:])
                 sel['Key']        = "%d.%d" % (fileIndex+1, entryIndex+1)
-                sel['legend']     = os.path.basename(sel['SourceName'][0])+\
+                sel['legend']     = os.path.basename(filename)+\
                                     " " + posixpath.basename(entry) #it was sel['Key']
                 sel['selection'] = {}
                 sel['selection']['sourcename'] = filename
                 #deal with the case the "entry" is a dataset hunging at root level
                 if isinstance(phynxFile[entry], h5py.Dataset):
+                    _logger.info("HDF5 dataset at root level")
+                    entry = "/"
+                elif hasattr(phynxFile[entry], "shape"):
+                    # HDF5-like dataset at top level
+                    _logger.info("HDF5-like dataset at root level")
                     entry = "/"
                 sel['selection']['entry'] = entry
                 sel['selection']['key'] = "%d.%d" % (fileIndex+1, entryIndex+1)
                 sel['selection']['x'] = cntSelection['x']
+                sel['selection']['xselectiontype'] = cntSelection['xselectiontype']
                 sel['selection']['y'] = [yCnt]
+                sel['selection']['yselectiontype'] = [cntSelection['yselectiontype'][cntSelection['y'].index(yCnt)]]
                 sel['selection']['m'] = cntSelection['m']
+                sel['selection']['mselectiontype'] = cntSelection['monselectiontype']
                 sel['selection']['cntlist'] = cntSelection['cntlist']
                 sel['selection']['LabelNames'] = cntSelection['aliaslist']
                 #sel['selection']['aliaslist'] = cntSelection['aliaslist']
@@ -966,10 +1107,20 @@ class QNexusWidget(qt.QWidget):
                     else:
                         actualDatasetPath = posixpath.join(entry,
                                                 cntSelection['cntlist'][yCnt])
-                    actualDataset = phynxFile[actualDatasetPath]
+                    _logger.info("actualDatasetPath = %s" % actualDatasetPath)
+                    try:
+                        actualDataset = phynxFile[actualDatasetPath]
+                    except KeyError:
+                        # filter x.1 and x.2 ESRF case
+                        if len(entryList) > 4:
+                            _logger.info("Ignoring %s in %s" % \
+                                         (actualDatasetPath, entry))
+                            continue
+                        else:
+                            raise
                     sel['scanselection'] = True
                     if hasattr(actualDataset, "shape"):
-                        if len(actualDataset.shape) > 1:
+                        if len(actualDataset.shape) > 1 and sel['selection']['yselectiontype'][0] in ["full", ""]:
                             if 1 in actualDataset.shape[-2:]:
                                 #shape (1, n) or (n, 1)
                                 pass
@@ -977,6 +1128,17 @@ class QNexusWidget(qt.QWidget):
                                 # at least twoD dataset
                                 selectionType= "2D"
                                 sel['scanselection'] = False
+                        elif len(actualDataset.shape) > 1 and sel['selection']['yselectiontype'][0].startswith("index"):
+                            if len(actualDataset.shape) <= 2:
+                                # 1D selection
+                                pass
+                            else:
+                                # at least twoD dataset
+                                selectionType= "2D"
+                                sel['scanselection'] = False
+                        elif len(actualDataset.shape) > 1 and sel['selection']['yselectiontype'][0].startswith("slice"):
+                            # 1D selection
+                            pass
                     n_axes = len(sel['selection']['x'])
                     if n_axes > 1:
                         selectionType = "%dD" % len(sel['selection']['x'])
@@ -1024,22 +1186,36 @@ class QNexusWidget(qt.QWidget):
                     actualDataset = phynxFile[actualDatasetPath]
                     if hasattr(actualDataset, "shape"):
                         actualDatasetLen = len(actualDataset.shape)
-                        if (actualDatasetLen == 2) and (1 in actualDataset.shape):
-                            # still can be used
-                            pass
-                        elif (actualDatasetLen > 1) and (not hasattr(self, "_messageShown")):
-                            # at least twoD dataset
-                            msg = qt.QMessageBox(self)
-                            msg.setIcon(qt.QMessageBox.Information)
-                            msg.setText("Multidimensional data set as MCA. Using Average. You should use ROI Imaging")
-                            msg.exec_()
-                            self._messageShown = True
-                        sel['selection']['mcaselectiontype'] = "avg"
+                        if sel['selection']['yselectiontype'][0] not in ["", "full", None]:
+                            sel['selection']['mcaselectiontype'] = sel['selection']['yselectiontype'][0]
+                        else:
+                            if (actualDatasetLen == 2) and (1 in actualDataset.shape):
+                                # still can be used
+                                pass
+                            elif (actualDatasetLen > 1) and (not hasattr(self, "_messageShown")):
+                                # at least twoD dataset
+                                msg = qt.QMessageBox(self)
+                                msg.setIcon(qt.QMessageBox.Information)
+                                msg.setText("Multidimensional data set as MCA. Using Average.\n Consider slicing or use ROI Imaging tool")
+                                msg.exec()
+                                self._messageShown = True
+                            sel['selection']['mcaselectiontype'] = "avg"
                 else:
                     sel['scanselection'] = False
                     sel['mcaselection']  = False
                 sel['selection']['selectiontype'] = selectionType
                 aliases = cntSelection['aliaslist']
+                if sel['selection']['yselectiontype'][0] not in ["", "full", None]:
+                     aliases[yCnt] += " " + sel['selection']['yselectiontype'][0]
+
+                if len(cntSelection['m']):
+                     if sel['selection']['mselectiontype'][0] not in ["", "full", None]:
+                         aliases[cntSelection['m'][0]] += " " + sel['selection']['mselectiontype'][0]
+
+                if len(cntSelection['x']):
+                     if sel['selection']['xselectiontype'][0] not in ["", "full", None]:
+                         aliases[cntSelection['x'][0]] += " " + sel['selection']['xselectiontype'][0]
+
                 if len(cntSelection['x']) and len(cntSelection['m']):
                     addLegend = " (%s/%s) vs %s" % (aliases[yCnt],
                                                    aliases[cntSelection['m'][0]],
@@ -1059,6 +1235,10 @@ class QNexusWidget(qt.QWidget):
             return selectionList
         self._lastAction = "%s" % ddict['action']
         if len(selectionList):
+            if text.upper() == "MCA":
+                self._lastMcaSelection = mcaSelection
+            else:
+                self._lastCntSelection = cntSelection
             if selectionType.upper() in ["SCAN", "MCA"]:
                 ddict = {}
                 ddict['event'] = "SelectionTypeChanged"
@@ -1103,7 +1283,7 @@ class QNexusWidget(qt.QWidget):
                             try:
                                 widget = self._widgetDict[ddict['id']]
                                 _logger.debug("DELETING %s", widget.windowTitle())
-                            except:
+                            except Exception:
                                 pass
                         del self._widgetDict[ddict['id']]
 
@@ -1113,8 +1293,8 @@ if __name__ == "__main__":
     app = qt.QApplication(sys.argv)
     try:
         #this is to add the 3D buttons ...
-        from PyMca5 import Object3D
-    except:
+        from PyMca5.PyMcaGui.pymca import SilxGLWindow
+    except Exception:
         pass
     w = QNexusWidget()
     if 0:
@@ -1133,5 +1313,5 @@ if __name__ == "__main__":
     w.sigAddSelection.connect(addSelection)
     w.sigRemoveSelection.connect(removeSelection)
     w.sigReplaceSelection.connect(replaceSelection)
-    ret = app.exec_()
+    ret = app.exec()
     sys.exit(ret)
