@@ -1,8 +1,8 @@
 #/*##########################################################################
-# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2024 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
-# the ESRF by the Software group.
+# the ESRF.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 # THE SOFTWARE.
 #
 #############################################################################*/
-__author__ = "V.A. Sole - ESRF Data Analysis"
+__author__ = "V.A. Sole - ESRF"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
@@ -185,12 +185,13 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
                 bins[1] = 2
         else:
             bins = self._bins
-        x0 = x.min()
-        y0 = y.min()
-        image = numpy.histogram2d(y, x,
+        idx = numpy.where(numpy.isfinite(x) & numpy.isfinite(y))
+        x0 = x[idx].min()
+        y0 = y[idx].min()
+        image = numpy.histogram2d(y[idx], x[idx],
                                   bins=bins,
                                   #range=(binsY, binsX),
-                                  normed=False)
+                                  density=False)
         self._binsX = image[2]
         self._binsY = image[1]
         self._bins = bins
@@ -230,15 +231,16 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
                 bins[1] = 2
         else:
             bins = self._bins
-        x0 = x.min()
-        y0 = y.min()
-        deltaX = (x.max() - x0) / float(bins[0] - 1)
-        deltaY = (y.max() - y0) / float(bins[1] - 1)
+        idx = numpy.where(numpy.isfinite(x) & numpy.isfinite(y))
+        x0 = x[idx].min()
+        y0 = y[idx].min()
+        deltaX = (x[idx].max() - x0) / float(bins[0] - 1)
+        deltaY = (y[idx].max() - y0) / float(bins[1] - 1)
         self.xScale = (x0, deltaX)
         self.yScale = (y0, deltaY)
         binsX = numpy.arange(bins[0]) * deltaX
         binsY = numpy.arange(bins[1]) * deltaY
-        image = numpy.histogram2d(y, x, bins=(binsY, binsX), normed=False)
+        image = numpy.histogram2d(y[idx], x[idx], bins=(binsY, binsX), density=False)
         self._binsX = image[2]
         self._binsY = image[1]
         self._bins = bins
@@ -253,16 +255,16 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
                     # this does not work properly yet
                     weightsSum = weights.sum(dtype=numpy.float64)
                     volume = (binsY[1] - binsY[0]) * (binsX[1] - binsX[0])
-                    mask = numpy.round(numpy.histogram2d(y, x,
+                    mask = numpy.round(numpy.histogram2d(y[idx], x[idx],
                                        bins=(binsY, binsX),
                                        weights=weights,
-                                       normed=True)[0] * weightsSum * volume).astype(numpy.uint8)
+                                       density=True)[0] * weightsSum * volume).astype(numpy.uint8)
                 else:
                     #print("GOOD PATH")
-                    mask = numpy.histogram2d(y, x,
+                    mask = numpy.histogram2d(y[idx], x[idx],
                                              bins=(binsY, binsX),
                                              weights=weights,
-                                             normed=False)[0]
+                                             density=False)[0]
                     mask[mask > 0] = 1
                 #print(mask.min(), mask.max())
                 self._densityPlotWidget.setSelectionMask(mask, plot=False)
@@ -374,12 +376,14 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
     def setSelectionMask(self, mask=None, plot=True):
         if self._selectionCurve is not None:
             selectionCurve = self.getCurve(self._selectionCurve)
+        else:
+            selectionCurve = None
         if selectionCurve in [[], None]:
             self._selectionCurve = None
             self._selectionMask = mask
         else:
             x, y = selectionCurve[0:2]
-            x = numpy.array(x, copy=False)
+            x = numpy.asarray(x)
             if hasattr(mask, "size"):
                 if mask.size == x.size:
                     if self._selectionMask is None:
@@ -450,7 +454,9 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
                     color = self._selectionColors[i].copy()
                     if useAlpha:
                         if len(color) == 4:
-                            if type(color[3]) in [numpy.uint8, numpy.int]:
+                            if type(color[3]) in [numpy.uint8,
+                                                  numpy.int32,
+                                                  numpy.int64]:
                                 color[3] = self._alphaLevel
                     # a copy of the input info is needed in order not
                     # to set the main curve to that color
@@ -644,7 +650,7 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
                                        self._setBrush5)
             self._brushMenu.addAction(QString("20 Image Pixel Width"),
                                        self._setBrush6)
-        self._brushMenu.exec_(self.cursor().pos())
+        self._brushMenu.exec(self.cursor().pos())
 
     def _brushSelectionIconSignal(self):
         _logger.debug("_setBrushSelectionMode")
@@ -778,9 +784,9 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
 
     def _additionalIconSignal(self):
         if self._plotViewMode == "density": # and imageData is not none ...
-            self._additionalSelectionMenu["density"].exec_(self.cursor().pos())
+            self._additionalSelectionMenu["density"].exec(self.cursor().pos())
         else:
-            self._additionalSelectionMenu["scatter"].exec_(self.cursor().pos())
+            self._additionalSelectionMenu["scatter"].exec(self.cursor().pos())
 
     def __resetSelection(self):
         # Needed because receiving directly in _resetSelection it was passing
@@ -926,7 +932,7 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
                 column = int((x[i] - x0) /deltaX)
                 try:
                     value = densityPlotMask[row, column]
-                except:
+                except Exception:
                     if row >= densityPlotMask.shape[0]:
                         row = densityPlotMask.shape[0] - 1
                     if column >= densityPlotMask.shape[1]:
@@ -997,6 +1003,7 @@ if __name__ == "__main__":
         print("Received: ", ddict)
     x = numpy.arange(100.)
     y = x * 1
+    y[50] = numpy.nan
     w = MaskScatterWidget(maxNRois=10, bins=(100,100), backend=backend)
     w.setSelectionCurveData(x, y, color="k", selectable=False)
     import numpy.random
@@ -1004,5 +1011,5 @@ if __name__ == "__main__":
     w.setPolygonSelectionMode()
     w.sigMaskScatterWidgetSignal.connect(receivingSlot)
     w.show()
-    app.exec_()
+    app.exec()
 
