@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2019 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2023 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
-# the ESRF by the Software group.
+# the ESRF.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 # THE SOFTWARE.
 #
 #############################################################################*/
-__author__ = "V.A. Sole - ESRF Data Analysis"
+__author__ = "V.A. Sole - ESRF"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
@@ -85,84 +85,55 @@ if __name__ == '__main__':
             else:
                 nativeFileDialogs = False
         elif opt in ('--PySide'):
-            print("Please use --binding=PySide")
-            import PySide.QtCore
+            print("Please use --binding=PySide6")
+            import PySide6.QtCore
         elif opt in ('--binding'):
             binding = arg.lower()
             if binding == "pyqt5":
                 import PyQt5.QtCore
-            elif binding == "pyqt4":
-                if sys.version_info < (3,):
-                    try:
-                        import sip
-                        sip.setapi('QString', 2)
-                        sip.setapi('QVariant', 2)
-                        sip.setapi('QDate', 2)
-                        sip.setapi('QDateTime', 2)
-                        sip.setapi('QTextStream', 2)
-                        sip.setapi('QTime', 2)
-                        sip.setapi('QUrl', 2)
-                    except:
-                        print("Cannot set sip API")
-                import PyQt4.QtCore
             elif binding == "pyside2":
                 import PySide2.QtCore
-            elif binding == "pyside":
-                import PySide.QtCore
+            elif binding == "pyside6":
+                import PySide6.QtCore
+            elif binding == "pyqt6":
+                import PyQt6.QtCore
             else:
                 raise ValueError("Unknown Qt binding <%s>" % binding)
 
     from PyMca5.PyMcaCore.LoggingLevel import getLoggingLevel
     logging.basicConfig(level=getLoggingLevel(opts))
-    if "HDF5_USE_FILE_LOCKING" not in os.environ:
-        if "h5py" in sys.modules:
-             _logger.warning("h5py already imported")
-        os.environ["HDF5_USE_FILE_LOCKING"] = "FALSE"
-        _logger.info("%s set to %s" % ("HDF5_USE_FILE_LOCKING",
-                                       os.environ["HDF5_USE_FILE_LOCKING"]))
+    # We are going to read. Disable file locking.
+    os.environ["HDF5_USE_FILE_LOCKING"] = "FALSE"
+    _logger.info("%s set to %s" % ("HDF5_USE_FILE_LOCKING",
+                                    os.environ["HDF5_USE_FILE_LOCKING"]))
     if binding is None:
         if qtversion == '3':
             raise NotImplementedError("Qt3 is no longer supported")
         elif qtversion == '4':
-            if sys.version_info < (3,):
-                try:
-                    import sip
-                    sip.setapi('QString', 2)
-                    sip.setapi('QVariant', 2)
-                    sip.setapi('QDate', 2)
-                    sip.setapi('QDateTime', 2)
-                    sip.setapi('QTextStream', 2)
-                    sip.setapi('QTime', 2)
-                    sip.setapi('QUrl', 2)
-                except:
-                    print("Cannot set sip API")
-            import PyQt4.QtCore
+            raise NotImplementedError("Qt4 is no longer supported")
         elif qtversion == '5':
-            import PyQt5.QtCore
-
+            try:
+                import PyQt5.QtCore
+            except ImportError:
+                import PySide2.QtCore
+        elif qtversion == '6':
+            import PySide6.QtCore
     try:
         # make sure hdf5plugins are imported
         import hdf5plugin
-    except:
+    except Exception:
         _logger.info("Failed to import hdf5plugin")
 
 from PyMca5.PyMcaGui import PyMcaQt as qt
+from PyMca5.PyMcaGui.io import PyMcaFileDialogs
 QTVERSION = qt.qVersion()
-if sys.platform == 'darwin':
-    if backend is not None:
-        if backend.lower() in ["gl", "opengl"]:
-            if hasattr(qt, 'QOpenGLWidget'):
-                print("Warning: OpenGL backend not fully supported")
+
 try:
     import silx
-    if (sys.version_info < (3,)) and ("PyQt4.QtCore" in sys.modules):
-        # PyQt4 is the most reliable binding at the ESRF for python 2
-        silxLogger = logging.getLogger("silx.DEPRECATION")
-        silxLogger.setLevel(logging.CRITICAL)
     # try to import silx prior to importing matplotlib to prevent
     # unnecessary warning
     import silx.gui.plot
-except:
+except Exception:
     pass
 from PyMca5.PyMcaGui.pymca import PyMcaMdi
 IconDict = PyMcaMdi.IconDict
@@ -175,13 +146,13 @@ else:
 try:
     from PyMca5.PyMcaGui.physics.xrf import XRFMCPyMca
     XRFMC_FLAG = True
-except:
+except Exception:
     XRFMC_FLAG = False
 
 try:
     from PyMca5.PyMcaGui.pymca import SumRulesTool
     SUMRULES_FLAG = True
-except:
+except Exception:
     SUMRULES_FLAG = False
 
 # prefer lazy import to avoid OpenCL related crashes on startup
@@ -229,7 +200,7 @@ if __name__ == "__main__":
         splash.showMessage( 'PyMca %s' % __version__,
                 alignment,
                 qt.Qt.white)
-    except:
+    except Exception:
         # fall back to original implementation in case of troubles
         splash.showMessage( 'PyMca %s' % __version__,
                 qt.Qt.AlignLeft|qt.Qt.AlignBottom,
@@ -249,19 +220,17 @@ try:
     #This is to make sure it is properly frozen
     #and that Object3D is fully supported
     import OpenGL.GL
-    #import Object3D.SceneGLWindow as SceneGLWindow
-    from PyMca5.PyMcaGui.pymca import PyMcaGLWindow
     OBJECT3D = False
     if ("PyQt4.QtOpenGL" in sys.modules) or \
        ("PySide.QtOpenGL") in sys.modules or \
+       ("PySide6.QtOpenGL") in sys.modules or \
        ("PySide2.QtOpenGL") in sys.modules or \
        ("PyQt5.QtOpenGL") in sys.modules:
         OBJECT3D = True
-except:
+except Exception:
     _logger.info("pyopengl not installed")
     OBJECT3D = False
 
-
 # Silx OpenGL availability (former Object3D)
 try:
     import PyMca5.PyMcaGui.pymca.SilxGLWindow
@@ -271,16 +240,14 @@ except ImportError:
 
 if isSilxGLAvailable:
     SceneGLWindow = PyMca5.PyMcaGui.pymca.SilxGLWindow
-elif OBJECT3D:
-    SceneGLWindow = PyMcaGLWindow
+else:
+    SceneGLWindow = None
 
 try:
     from PyMca5.PyMcaGui.pymca import SilxScatterWindow
-except:
+except Exception:
     _logger.info("Cannot import SilxScatterWindow")
 
-_logger.info("SilxGL availability: %s", isSilxGLAvailable)
-
 # check that OpenGL is actually supported (ESRF rnice problems)
 OPENGL_DRIVERS_OK = True
 if sys.platform.startswith("linux"):
@@ -292,9 +259,27 @@ if sys.platform.startswith("linux"):
             OBJECT3D = False
             _logger.warning("OpenGL disabled. Errors using glxinfo command")
 
+    if OPENGL_DRIVERS_OK and isSilxGLAvailable:
+        # additional test (takes care of disabled forwarding)
+        try:
+            from silx.gui.utils.glutils import isOpenGLAvailable
+            isSilxGLAvailable = isOpenGLAvailable(version=(2, 1), runtimeCheck=True)
+            if not isSilxGLAvailable:
+                _logger.info("OpenGL >= 2.1 not available")
+                if not isOpenGLAvailable(version=(1, 4), runtimeCheck=True):
+                    _logger.info("OpenGL >= 1.4 not available")
+                    OPENGL_DRIVERS_OK = False
+                    OBJECT3D = False
+        except Exception:
+            _logger.info("Cannot test OpenGL availability %s" % sys.exc_info()[1])
+
+# PyMca 3D disabled in favor of silx
+OBJECT3D = isSilxGLAvailable
+_logger.info("SilxGL availability: %s", isSilxGLAvailable)
+
 from PyMca5.PyMcaGui.pymca import QDispatcher
-from PyMca5.PyMcaGui import ElementsInfo
-from PyMca5.PyMcaGui import PeakIdentifier
+from PyMca5.PyMcaGui.physics.xrf import ElementsInfo
+from PyMca5.PyMcaGui.physics.xrf import PeakIdentifier
 from PyMca5.PyMcaGui.pymca import PyMcaBatch
 ###########import Fit2Spec
 from PyMca5.PyMcaGui.pymca import Mca2Edf
@@ -302,11 +287,11 @@ try:
     from PyMca5.PyMcaGui.pymca import QStackWidget
     from PyMca5.PyMcaGui.pymca import StackSelector
     STACK = True
-except:
+except Exception:
     STACK = False
 from PyMca5.PyMcaGui.pymca import PyMcaPostBatch
-from PyMca5.PyMcaGui import RGBCorrelator
-from PyMca5.PyMcaGui import MaterialEditor
+from PyMca5.PyMcaGui.pymca import RGBCorrelator
+from PyMca5.PyMcaGui.physics.xrf import MaterialEditor
 
 from PyMca5.PyMcaIO import ConfigDict
 from PyMca5 import PyMcaDirs
@@ -316,7 +301,7 @@ if QTVERSION > '4.3.0':
     try:
         from PyMca5.PyMcaCore import XiaCorrect
         XIA_CORRECT = True
-    except:
+    except Exception:
         pass
 
 SOURCESLIST = QDispatcher.QDataSource.source_types.keys()
@@ -327,7 +312,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 fl = qt.Qt.WA_DeleteOnClose
             PyMcaMdi.PyMcaMdi.__init__(self, parent, name, fl)
             maxheight = qt.QDesktopWidget().height()
-            if maxheight < 799:
+            if maxheight < 799 and maxheight > 0:
                 self.setMinimumHeight(int(0.8*maxheight))
                 self.setMaximumHeight(int(0.9*maxheight))
             self.setWindowTitle(name)
@@ -421,7 +406,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             try:
                 defaultFileName = PyMca5.getDefaultSettingsFile()
                 self.configDir  = os.path.dirname(defaultFileName)
-            except:
+            except Exception:
                 if not ('fresh' in kw):
                     raise
             if not ('fresh' in kw):
@@ -537,7 +522,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
 
         try:
             self._dispatcherAddSelectionSlot(ddict)
-        except:
+        except Exception:
             if _logger.getEffectiveLevel() == logging.DEBUG:
                 raise
             msg = qt.QMessageBox(self)
@@ -545,7 +530,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             msg.setText("Error: %s" % sys.exc_info()[1])
             msg.setInformativeText(str(sys.exc_info()[1]))
             msg.setDetailedText(traceback.format_exc())
-            msg.exec_()
+            msg.exec()
 
     def _dispatcherAddSelectionSlot(self, dictOrList):
         _logger.info("self._dispatcherAddSelectionSlot(ddict), ddict = %s",
@@ -603,7 +588,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                         if ('tth' in motor_mne) and ('th' in motor_mne):
                             #FOURC
                             hkl = True
-                except:
+                except Exception:
                     pass
                 if hkl:
                     imageWindow = PyMcaHKLImageWindow.PyMcaHKLImageWindow(
@@ -614,7 +599,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                     imageWindow = PyMcaImageWindow.PyMcaImageWindow(
                             name=legend,
                             correlator=self.imageWindowCorrelator,
-                            scanwindow=self.scanWindow)
+                            scanwindow=self.scanWindow,
+                            usesilx=None)
                 self.imageWindowDict[legend] = imageWindow
 
                 imageWindow.sigAddImageClicked.connect( \
@@ -668,12 +654,13 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
     def dispatcherRemoveSelectionSlot(self, ddict):
         try:
             return self._dispatcherRemoveSelectionSlot(ddict)
-        except:
+        except Exception:
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Critical)
-            msg.setText("Error: %s" % sys.exc_info()[1])
-            msg.exec_()
-
+            txt = "Error: %s" % sys.exc_info()[1]
+            msg.setInformativeText(txt)
+            msg.setDetailedText(traceback.format_exc())
+            msg.exec()
 
     def _dispatcherRemoveSelectionSlot(self, dictOrList):
         _logger.debug("self.dispatcherRemoveSelectionSlot(ddict), ddict = %s", dictOrList)
@@ -681,7 +668,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             ddict = dictOrList[0]
         else:
             ddict = dictOrList
-        if self._isScatterSelection(ddict) or self._is2DSelection(ddict):
+        if self._is2DSelection(ddict) or self._isScatterSelection(ddict):
             legend = ddict['legend']
             if legend in self.imageWindowDict.keys():
                 index = self.mainTabWidget.indexOf(self.imageWindowDict[legend])
@@ -700,11 +687,13 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
     def dispatcherReplaceSelectionSlot(self, ddict):
         try:
             return self._dispatcherReplaceSelectionSlot(ddict)
-        except:
+        except Exception:
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Critical)
-            msg.setText("Error: %s" % sys.exc_info()[1])
-            msg.exec_()
+            txt = "Error: %s" % sys.exc_info()[1]
+            msg.setInformativeText(txt)
+            msg.setDetailedText(traceback.format_exc())
+            msg.exec()
 
     def _dispatcherReplaceSelectionSlot(self, dictOrList):
         _logger.debug("self.dispatcherReplaceSelectionSlot(ddict), ddict = %s",
@@ -876,7 +865,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         try:
             d['ScanSimpleFit']['Configuration'].update(
                 self.scanWindow.scanFit.getConfiguration())
-        except:
+        except Exception:
             if _logger.getEffectiveLevel() == logging.DEBUG:
                 raise
             _logger.warning("Error getting ScanFit configuration")
@@ -937,6 +926,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                         try:
                             PyMcaDirs.inputDir = ddict[source] ['lastInputDir']
                         except ValueError:
+                            _logger.info("Cannot open directory <%s>" % ddict[source] ['lastInputDir'])
                             pass
                 if 'SourceName' in ddict[source]:
                     if type(ddict[source]['SourceName']) != type([]):
@@ -962,26 +952,26 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                                     self.sourceWidget.selectorWidget[source].openFile(SourceName, justloaded=1)
                                 else:
                                     self.sourceWidget.selectorWidget[source].openFile(SourceName)
-                            except:
+                            except Exception:
                                 msg = qt.QMessageBox(self)
                                 msg.setIcon(qt.QMessageBox.Critical)
                                 txt = "Error: %s\n opening file %s" % (sys.exc_info()[1],SourceName )
                                 msg.setInformativeText(txt)
                                 msg.setDetailedText(traceback.format_exc())
-                                msg.exec_()
+                                msg.exec()
 
                 if 'WidgetConfiguration' in ddict[source]:
                     selectorWidget = self.sourceWidget.selectorWidget[source]
                     if hasattr(selectorWidget,'setWidgetConfiguration'):
                         try:
                             selectorWidget.setWidgetConfiguration(ddict[source]['WidgetConfiguration'])
-                        except:
+                        except Exception:
                             msg = qt.QMessageBox(self)
                             msg.setIcon(qt.QMessageBox.Critical)
                             txt = "Error: %s\n configuring %s widget" % (sys.exc_info()[1], source )
                             msg.setInformativeText(txt)
                             msg.setDetailedText(traceback.format_exc())
-                            msg.exec_()
+                            msg.exec()
         if "McaWindow" in ddict:
             self.mcaWindow.setCalibrations(ddict["McaWindow"]["calibrations"])
 
@@ -1017,11 +1007,11 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                                               **d['Information'])
                 if d['LastFit']['hidden'] == 'False':
                     self.mcaWindow.advancedfit.show()
-                    self.mcaWindow.advancedfit.raiseW()
+                    self.mcaWindow.advancedfit.raise_()
                     if d['LastFit']['fitdone']:
                         try:
                             self.mcaWindow.advancedfit.fit()
-                        except:
+                        except Exception:
                             pass
                 else:
                     print("hidden")
@@ -1042,11 +1032,11 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                                               **d['Information'])
                 if d['LastFit']['hidden'] == 'False':
                     self.mcaWindow.advancedfit.show()
-                    self.mcaWindow.advancedfit.raiseW()
+                    self.mcaWindow.advancedfit.raise_()
                     if d['LastFit']['fitdone']:
                         try:
                             self.mcaWindow.advancedfit.fit()
-                        except:
+                        except Exception:
                             pass
                 else:
                     _logger.info("hidden")
@@ -1065,21 +1055,21 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.actionOpen = qt.QAction(self)
             self.actionOpen.setText(QString("&Open"))
             self.actionOpen.setIcon(self.Icons["fileopen"])
-            self.actionOpen.setShortcut(qt.QKeySequence(qt.Qt.CTRL+qt.Qt.Key_O))
+            self.actionOpen.setShortcut(qt.QKeySequence(qt.Qt.CTRL|qt.Qt.Key_O))
             self.actionOpen.triggered[bool].connect(self.onOpen)
             #filesaveas
             self.actionSaveAs = qt.QAction(self)
             self.actionSaveAs.setText(QString("&Save"))
             self.actionSaveAs.setIcon(self.Icons["filesave"])
             self.actionSaveAs.setShortcut(\
-                qt.QKeySequence(qt.Qt.CTRL+qt.Qt.Key_S))
+                qt.QKeySequence(qt.Qt.CTRL|qt.Qt.Key_S))
             self.actionSaveAs.triggered[bool].connect(self.onSaveAs)
 
             #filesave
             self.actionSave = qt.QAction(self)
             self.actionSave.setText(QString("Save &Default Settings"))
             #self.actionSave.setIcon(self.Icons["filesave"])
-            #self.actionSave.setShortcut(qt.Qt.CTRL+qt.Qt.Key_S)
+            #self.actionSave.setShortcut(qt.Qt.CTRL|qt.Qt.Key_S)
             self.actionSave.triggered[bool].connect(self.onSave)
 
             #fileprint
@@ -1087,7 +1077,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.actionPrint.setText(QString("&Print"))
             self.actionPrint.setIcon(self.Icons["fileprint"])
             self.actionPrint.setShortcut(\
-                qt.QKeySequence(qt.Qt.CTRL+qt.Qt.Key_P))
+                qt.QKeySequence(qt.Qt.CTRL|qt.Qt.Key_P))
             self.actionPrint.triggered[bool].connect(self.onPrint)
 
             #filequit
@@ -1095,7 +1085,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.actionQuit.setText(QString("&Quit"))
             #self.actionQuit.setIcon(self.Icons["fileprint"])
             self.actionQuit.setShortcut(\
-                                       qt.QKeySequence(qt.Qt.CTRL+qt.Qt.Key_Q))
+                                       qt.QKeySequence(qt.Qt.CTRL|qt.Qt.Key_Q))
             qApp = qt.QApplication.instance()
             self.actionQuit.triggered.connect(qApp.closeAllWindows)
 
@@ -1133,6 +1123,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             self.menuHelp.addAction("MCA &HOWTOs",self.onMcaHowto)
             self.menuHelp.addSeparator()
             self.menuHelp.addAction("&About", self.onAbout)
+            self.menuHelp.addAction("Web Documentation", self.onWebDocumentation)
             self.menuHelp.addAction("Changes", self.onChanges)
             self.menuHelp.addAction("About &Qt",self.onAboutQt)
             self.menuBar().addSeparator()
@@ -1186,7 +1177,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         else:
             try:
                 self.splitter.insertWidget(0, self.sourceFrame)
-            except:
+            except Exception:
                 self.sourceFrame.setParent(self.splitter)
 
     def initSource(self):
@@ -1322,11 +1313,10 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                         "All Files (*)"]
         message = "Open ONE Batch result .dat file or SEVERAL EDF files"
         filelist = self.__getStackOfFiles(fileTypeList, message)
-        if not(len(filelist)):
-            return
-        filelist.sort()
-        self.sourceWidget.sourceSelector.lastInputDir = os.path.dirname(filelist[0])
-        PyMcaDirs.inputDir = os.path.dirname(filelist[0])
+        if filelist:
+            filelist.sort()
+            self.sourceWidget.sourceSelector.lastInputDir = os.path.dirname(filelist[0])
+            PyMcaDirs.inputDir = os.path.dirname(filelist[0])
         self.__correlator.append(PyMcaPostBatch.PyMcaPostBatch())
         for correlator in self.__correlator:
             if correlator.isHidden():
@@ -1334,7 +1324,8 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             correlator.raise_()
         self.__correlator[-1].sigRGBCorrelatorSignal.connect( \
                 self._deleteCorrelator)
-        correlator.addFileList(filelist)
+        if filelist:
+            correlator.addFileList(filelist)
 
     def __sumRules(self):
         if self.__correlator is None:
@@ -1351,7 +1342,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 txt = "This functionality requires tomogui, silx and FreeART"
                 msg.setInformativeText(txt)
                 msg.setDetailedText(traceback.format_exc())
-                msg.exec_()
+                msg.exec()
                 return
             self.__reconsWidget = TomoguiProjectWindow()
         if self.__reconsWidget.isHidden():
@@ -1371,69 +1362,13 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         wdir = PyMcaDirs.inputDir
         fileTypeList = typelist
         filterused = None
-        if False and PyMcaDirs.nativeFileDialogs:
-            #windows cannot handle thousands of files in a file dialog
-            filetypes = ""
-            for filetype in fileTypeList:
-                filetypes += filetype+"\n"
-            filelist = qt.QFileDialog.getOpenFileNames(self,
-                        message,
-                        wdir,
-                        filetypes)
-            if not len(filelist):
-                if getfilter:
-                    return [], filterused
-                else:
-                    return []
-            else:
-                sample  = qt.safe_str(filelist[0])
-                for filetype in fileTypeList:
-                    ftype = filetype.replace("(", "").replace(")","")
-                    extensions = ftype.split()[2:]
-                    for extension in extensions:
-                        if sample.endswith(extension[-3:]):
-                            filterused = filetype
-                            break
-        else:
-            fdialog = qt.QFileDialog(self)
-            fdialog.setModal(True)
-            fdialog.setWindowTitle(message)
-            if hasattr(qt, "QStringList"):
-                strlist = qt.QStringList()
-            else:
-                strlist = []
-            for filetype in fileTypeList:
-                strlist.append(filetype.replace("(","").replace(")",""))
-            if hasattr(fdialog, "setFilters"):
-                fdialog.setFilters(strlist)
-            else:
-                fdialog.setNameFilters(strlist)
-            fdialog.setFileMode(fdialog.ExistingFiles)
-            fdialog.setDirectory(wdir)
-            if QTVERSION > '4.3.0':
-                history = fdialog.history()
-                if len(history) > 6:
-                    fdialog.setHistory(history[-6:])
-            ret = fdialog.exec_()
-            if ret == qt.QDialog.Accepted:
-                filelist = fdialog.selectedFiles()
-                if getfilter:
-                    filterused = qt.safe_str(fdialog.selectedFilter())
-                fdialog.close()
-                del fdialog
-            else:
-                fdialog.close()
-                del fdialog
-                if getfilter:
-                    return [], filterused
-                else:
-                    return []
-        filelist = [qt.safe_str(x) for x in filelist]
+        #windows cannot handle thousands of files in a file dialog force Qt dialogs
+        output = PyMcaFileDialogs.getFileList(self, filetypelist=typelist, message=message, currentdir=wdir,
+                                        mode="OPEN", getfilter=getfilter, single=False, currentfilter=None, native=False)
         if getfilter:
-            return filelist, filterused
+            return output[0], output[1]
         else:
-            return filelist
-
+            return output
 
     def __roiImaging(self):
         if self.__imagingTool is None:
@@ -1441,7 +1376,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
             try:
                 widget = QStackWidget.QStackWidget(mcawidget=self.mcaWindow,
                                                    rgbwidget=rgbWidget,
-                                                   master=True)
+                                                   primary=True)
                 widget.notifyCloseEventToWidget(self)
                 self.__imagingTool = id(widget)
                 self._widgetDict[self.__imagingTool] = widget
@@ -1458,9 +1393,9 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 txt = "Input Output Error: %s" % (sys.exc_info()[1])
                 msg.setInformativeText(txt)
                 msg.setDetailedText(traceback.format_exc())
-                msg.exec_()
+                msg.exec()
                 return
-            except:
+            except Exception:
                 widget = None
                 del self._widgetDict[self.__imagingTool]
                 self.__imagingTool = None
@@ -1469,7 +1404,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 txt = "Error info = %s" % (sys.exc_info()[1])
                 msg.setInformativeText(txt)
                 msg.setDetailedText(traceback.format_exc())
-                msg.exec_()
+                msg.exec()
                 return
         else:
             widget = self._widgetDict[self.__imagingTool]
@@ -1498,7 +1433,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                     if id(widget) != id(self):
                         if widget.parent() is None:
                             widget.close()
-                except:
+                except Exception:
                     _logger.debug("Error closing widget")
             from PyMca5.PyMcaGui.plotting import PyMcaPrintPreview
             PyMcaPrintPreview.resetSingletonPrintPreview()
@@ -1521,7 +1456,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         self._sumRulesTool.raise_()
 
     def onOpen(self):
-        self.openMenu.exec_(self.cursor().pos())
+        self.openMenu.exec(self.cursor().pos())
 
     def onSave(self):
         self._saveAs()
@@ -1540,43 +1475,31 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
         elif text in self.imageWindowDict.keys():
             self.saveMenu.addAction("Active Image",
                   self.imageWindowDict[text].graphWidget._saveIconSignal)
-        self.saveMenu.exec_(self.cursor().pos())
+        self.saveMenu.exec(self.cursor().pos())
 
     def _onSaveAs(self):
         cwd = os.getcwd()
-        outfile = qt.QFileDialog(self)
-        if hasattr(outfile, "setFilters"):
-            outfile.setFilters(['PyMca  *.ini'])
-        else:
-            outfile.setNameFilters(['PyMca  *.ini'])
-        outfile.setFileMode(outfile.AnyFile)
-        outfile.setAcceptMode(qt.QFileDialog.AcceptSave)
-
         if os.path.exists(self.configDir):
-            cwd =self.configDir
-        outfile.setDirectory(cwd)
-        ret = outfile.exec_()
-        if ret:
-            if hasattr(outfile, "selectedFilter"):
-                filterused = qt.safe_str(outfile.selectedFilter()).split()
-            else:
-                filterused = qt.safe_str(outfile.selectedNameFilter()).split()
-            extension = ".ini"
-            outdir=qt.safe_str(outfile.selectedFiles()[0])
-            try:
-                outputDir  = os.path.dirname(outdir)
-            except:
-                outputDir  = "."
-            try:
-                outputFile = os.path.basename(outdir)
-            except:
-                outputFile  = "PyMca.ini"
-            outfile.close()
-            del outfile
-        else:
-            outfile.close()
-            del outfile
+            cwd = self.configDir
+
+        outputFile = PyMcaFileDialogs.getFileList(self, filetypelist=['PyMca (*.ini)'],
+                                                message="Provide output file",
+                                                currentdir=cwd, mode="SAVE",
+                                                getfilter=False, single=False,
+                                                currentfilter=None, native=None)
+        if not len(outputFile):
             return
+        extension = ".ini"
+        outputFile = outputFile[0]
+        try:
+            outputDir  = os.path.dirname(outputFile)
+        except Exception:
+            outputDir  = "."
+        try:
+            outputFile = os.path.basename(outputFile)
+        except Exception:
+            outputFile  = "PyMca.ini"
+
         #always overwrite for the time being
         if len(outputFile) < len(extension[:]):
             outputFile += extension[:]
@@ -1592,16 +1515,16 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 txt = "Input Output Error: %s" % (sys.exc_info()[1])
                 msg.setInformativeText(txt)
                 msg.setDetailedText(traceback.format_exc())
-                msg.exec_()
+                msg.exec()
                 return
         try:
             self._saveAs(filename)
             self.configDir = outputDir
-        except:
+        except Exception:
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Critical)
             msg.setText("Error Saving Configuration: %s" % (sys.exc_info()[1]))
-            msg.exec_()
+            msg.exec()
             return
 
     def _saveAs(self, filename=None):
@@ -1621,7 +1544,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 self.trainingDataMenu.addAction(action)
         try:
             source = None
-            selectedAction = self.trainingDataMenu.exec_(qt.QCursor.pos())
+            selectedAction = self.trainingDataMenu.exec(qt.QCursor.pos())
             if selectedAction is not None:
                 key = qt.safe_str(selectedAction.text())
                 source = self.trainingSources[key]
@@ -1633,7 +1556,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                     # but we change the input dir to allow easy loading of the config file from the
                     # fit configuration window
                     PyMcaDirs.inputDir = os.path.dirname(source)
-        except:
+        except Exception:
             msg = qt.QMessageBox(self)
             msg.setIcon(qt.QMessageBox.Critical)
             msg.setWindowTitle("Error opening data source")
@@ -1643,29 +1566,27 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 msg.setText("Cannot open data source")
             msg.setInformativeText(str(sys.exc_info()[1]))
             msg.setDetailedText(traceback.format_exc())
-            msg.exec_()
+            msg.exec()
 
     def openSource(self,index=0):
         _logger.debug("index = %d ", index)
         if index <= 0:
-            outfile = qt.QFileDialog(self)
-            outfile.setWindowTitle("Select PyMca Configuration File")
             if os.path.exists(self.configDir):
-                outfile.setDirectory(self.configDir)
-            if hasattr(outfile, "setFilters"):
-                outfile.setFilters(['PyMca  *.ini'])
+                currentdir = self.configDir
             else:
-                outfile.setNameFilters(['PyMca  *.ini'])
-            outfile.setFileMode(outfile.ExistingFile)
-            ret = outfile.exec_()
-            if ret:
-                filename = qt.safe_str(outfile.selectedFiles()[0])
-                outfile.close()
-                del outfile
-            else:
-                outfile.close()
-                del outfile
+                currentdir = None
+            filename = PyMcaFileDialogs.getFileList(parent=self,
+                                                   filetypelist=['PyMca (*.ini)'],
+                                                   message="Select PyMca Configuration File",
+                                                   currentdir=currentdir,
+                                                   mode="OPEN",
+                                                   getfilter=False,
+                                                   single=False,
+                                                   currentfilter=None,
+                                                   native=None)
+            if not len(filename):
                 return
+            filename = filename[0]
             currentConfigDict = ConfigDict.ConfigDict()
             self.configDir  = os.path.dirname(filename)
             currentConfigDict.read(filename)
@@ -1734,6 +1655,11 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                 "PyMca Application\nVersion: "+__version__)
         #self.onDebug()
 
+    def onWebDocumentation(self):
+        import webbrowser
+        url = "http://www.silx.org/doc/PyMca/dev/"
+        webbrowser.open(url)
+
     def onChanges(self):
         if self.changeLog is None:
             self.changeLog = qt.QTextEdit()
@@ -1770,7 +1696,7 @@ class PyMcaMain(PyMcaMdi.PyMcaMdi):
                         _logger.debug("Module name = %s, %s",
                                       module.__name__,
                                       module.__revision__.replace("$", ""))
-            except:
+            except Exception:
                 pass
 
     def onPrint(self):
@@ -1847,7 +1773,7 @@ if 0:
             return "Graph %d"%(idx)
 
 class MyQTextBrowser(qt.QTextBrowser):
-    def setSource(self, name):
+    def myWrappedSource(self, name, ignored=None):
         if name == QString("./PyMCA.html") or ("PyMCA.html" in ("%s" % name)):
             if sys.platform == 'win32':
                 ddir=PyMcaDataDir.PYMCA_DOC_DIR
@@ -1858,17 +1784,28 @@ class MyQTextBrowser(qt.QTextBrowser):
                 return
             try:
                 self.report.show()
-            except:
+            except Exception:
                 self.report = qt.QTextBrowser()
                 self.report.setCaption(QString("PyMca Report"))
                 ddir=PyMcaDataDir.PYMCA_DOC_DIR
                 self.report.mimeSourceFactory().addFilePath(QString(ddir+"/HTML"))
                 self.report.mimeSourceFactory().addFilePath(QString(ddir+"/HTML/PyMCA_files"))
                 self.report.setSource(name)
-            if self.report.isHidden():self.report.show()
-            self.report.raiseW()
+            if self.report.isHidden():
+                self.report.show()
+            self.report.raise_()
         else:
-            qt.QTextBrowser.setSource(self, name)
+            if QTVERSION.startswith('6'):
+                qt.QTextBrowser.doSetSource(self, name)
+            else:
+                qt.QTextBrowser.setSource(self, name)
+
+    if QTVERSION.startswith('6'):
+        def doSetSource(self, name, ignored=None):
+            return self.myWrappedSource(name, ignored)
+    else:
+        def setSource(self, name):
+            return self.myWrappedSource(name)
 
 class Line(qt.QFrame):
     sigLineDoubleClickEvent = qt.pyqtSignal(object)
@@ -1909,21 +1846,21 @@ if __name__ == '__main__':
     try:
         for source in args:
             PyMcaMainWidgetInstance.sourceWidget.sourceSelector.openSource(source)
-    except:
+    except Exception:
         msg = qt.QMessageBox(PyMcaMainWidgetInstance)
         msg.setIcon(qt.QMessageBox.Critical)
         msg.setWindowTitle("Error opening data source")
         msg.setText("Cannot open data source %s" % source)
         msg.setInformativeText(str(sys.exc_info()[1]))
         msg.setDetailedText(traceback.format_exc())
-        msg.exec_()
+        msg.exec()
 
     if PROFILING:
-        profile.run('sys.exit(app.exec_())',"test")
+        profile.run('sys.exit(app.exec())',"test")
         p=pstats.Stats("test")
         p.strip_dirs().sort_stats(-1).print_stats()
     else:
-        ret = app.exec_()
+        ret = app.exec()
         app = None
         sys.exit(ret)
 
